// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdlib.h>
#include    "orb_cplus.h"
#include    "CosNaming_cplus.h"
#include    "EventService_cplus.h"

// ---- main function --------------------------------------------
int
main(
    int                   argc,
    char                  *argv[] )
{
    int                   current_argc = argc;
    CORBA::ORB_ptr        orb;
    CORBA::Environment    env;
    CORBA::Object_ptr     obj;
    const char            *p;

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_NameService, env );
        if ( obj == CosNaming::NamingContext::_nil() ) {
            printf( "CosNaming::NamingContext = NULL!\n" );
            return(1);
        }

        CosNaming::NamingContext_ptr    naming_context;
        naming_context = CosNaming::NamingContext::_narrow( obj );
        CORBA::release( obj );

        // ---- EventChannel 'pull::sample' get ----
        CosNaming::Name_ptr             name;
        CosNaming::NameComponent_var    *name_component;

        name_component = CosNaming::Name::allocbuf( 1 );
        name_component[0]->id   = (const CORBA::Char *)"pull::sample";
        name_component[0]->kind = (const CORBA::Char *)"";
        name = new CosNaming::Name( 1, 1, name_component, CORBA_TRUE );
        p = "resolve";
        obj = naming_context->resolve( *name, env );
        delete name;

        CosEventChannelAdmin::EventChannel_ptr    channel;
        channel = CosEventChannelAdmin::EventChannel::_narrow( obj );
        CORBA::release( obj );

        // ---- connect EventChannel ----
        CosEventChannelAdmin::ConsumerAdmin_ptr         admin;
        CosEventChannelAdmin::ProxyPullSupplier_ptr     proxy;

        p = "for_consumers";
        admin = channel->for_consumers( env );
        p = "obtain_pull_supplier";
        proxy = admin->obtain_pull_supplier( env );
        p = "connect_pull_consumer";
        proxy->connect_pull_consumer(
                        CosEventComm::PullConsumer::_nil(), env );

        CORBA::Any    *recv;
        try {
            p = "pull";
            recv = proxy->pull( env );
        } catch( CORBA::SystemException &se ) {
            printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                    p, se.id(), se.minor() );
            try {
                proxy->disconnect_pull_supplier( env );
            } catch( CORBA::Exception &e ) {}
            return(1);
        } catch( CORBA::Exception &e ) {
            printf( "Operation=%s, Exception=%s\n", p, e.id() );
            try {
                proxy->disconnect_pull_supplier( env );
            } catch( CORBA::Exception &e ) {}
            return(1);
        }

        CORBA::Char *message = (CORBA::Char *)recv->value();
        printf( "pull message : %s\n", message );
        delete recv;

        p = "disconnect_pull_supplier";
        proxy->disconnect_pull_supplier( env );

        CORBA::release( proxy );
        CORBA::release( admin );
        CORBA::release( channel );

    } catch( CORBA::SystemException   &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception   &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    return(0);
}

