// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdlib.h>
#include    <string.h>
#include    "orb_cplus.h"
#include    "CosNaming_cplus.h"
#include    "NotificationService_cplus.h"

// ---- main function --------------------------------------------
int
main(
    int                         argc,
    char                        *argv[] )
{
    int                         current_argc = argc;
    CORBA::ORB_ptr              orb;
    CORBA::Environment          env;
    CORBA::Object_ptr           obj;
    const char                  *p;

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_NameService, env );
        if ( obj == CosNaming::NamingContext::_nil() ) {
            printf( "CosNaming::NamingContext = NULL!\n" );
            return(1);
        }

        CosNaming::NamingContext_ptr    naming_context;
        naming_context = CosNaming::NamingContext::_narrow( obj );
        CORBA::release( obj );

        // ---- EventChannel 'mix::sample' get ----
        CosNaming::Name_ptr             name;
        CosNaming::NameComponent_var    *name_component;

        name_component = CosNaming::Name::allocbuf( 1 );
        name_component[0]->id   = (const CORBA::Char *)"mix::sample";
        name_component[0]->kind = (const CORBA::Char *)"";
        name = new CosNaming::Name( 1, 1, name_component, CORBA_TRUE );
        p = "resolve";
        obj = naming_context->resolve( *name, env );
        delete name;

        CosNotifyChannelAdmin::EventChannel_ptr    channel;
        channel = CosNotifyChannelAdmin::EventChannel::_narrow( obj );
        CORBA::release( obj );

        // ---- connect EventChannel ----
        CosNotifyChannelAdmin::SupplierAdmin_ptr         admin;
        CosNotifyChannelAdmin::ProxyConsumer_ptr         tmp_proxy;
        CosNotifyChannelAdmin::ProxyPushConsumer_ptr     proxy;
        CosNotifyChannelAdmin::ProxyID                   proxyID;

        p = "default_supplier_admin";
        admin = channel->default_supplier_admin( env );
        p = "obtain_notification_push_consumer";
        tmp_proxy = admin->obtain_notification_push_consumer(
                        CosNotifyChannelAdmin::ANY_EVENT, proxyID, env );
        proxy = CosNotifyChannelAdmin::ProxyPushConsumer::_narrow( tmp_proxy );
        CORBA::release( tmp_proxy );
        p = "connect_any_push_supplier";
        proxy->connect_any_push_supplier(
                        CosEventComm::PushSupplier::_nil(), env );

        CORBA::Any      *send = new CORBA::Any();
        CORBA::Char     *message;
        message = CORBA::string_alloc( strlen("Mixed-C++") );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
        strcpy_s( message, strlen("Mixed-C++")+1, "Mixed-C++" );
#else
        strcpy( message, "Mixed-C++" );
#endif
        send->replace( _tc_string, message, CORBA_TRUE );

        try {
            p = "push";
            proxy->push( *send, env );
        } catch( CORBA::SystemException &se ) {
            printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                    p, se.id(), se.minor() );
            try {
                proxy->disconnect_push_consumer( env );
            } catch( CORBA::Exception e ) {}
            return(1);
        } catch( CORBA::Exception &e ) {
            printf( "Operation=%s, Exception=%s\n", p, e.id() );
            try {
                proxy->disconnect_push_consumer( env );
            } catch( CORBA::Exception e ) {}
            return(1);
        }
        delete send;

        p = "disconnect_push_consumer";
        proxy->disconnect_push_consumer( env );
        CORBA::release( proxy );
        CORBA::release( admin );
        CORBA::release( channel );

    } catch( CORBA::SystemException   &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception   &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    return(0);
}

