// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdlib.h>
#include    "orb_cplus.h"
#include    "CosNaming_cplus.h"
#include    "NotificationService_cplus.h"

// ---- main function --------------------------------------------
int
main(
    int                 argc,
    char                *argv[] )
{
    int                 current_argc = argc;
    CORBA::ORB_ptr      orb;
    CORBA::Environment  env;
    CORBA::Environment  tmp_env;
    CORBA::Object_ptr   obj;
    const char          *p;
    ES::ChannelUtil_ptr util;

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_NameService, env );
        if ( obj == CosNaming::NamingContext::_nil() ) {
            printf( "CosNaming::NamingContext = NULL!\n" );
            return(1);
        }

        CosNaming::NamingContext_ptr    naming_context;
        naming_context = CosNaming::NamingContext::_narrow( obj );
        CORBA::release( obj );

        // ---- EventChannel 'mix::sample' get ----
        CosNaming::Name_ptr             name;
        CosNaming::NameComponent_var    *name_component;

        name_component = CosNaming::Name::allocbuf( 1 );
        if ( name_component == NULL ) {
            printf( "alloc error: CosNaming::Name::allocbuf()\n" );
            return(1);
        }
        name_component[0]->id   = (const CORBA::Char *)"mix::sample";
        name_component[0]->kind = (const CORBA::Char *)"";
        name = new CosNaming::Name( 1, 1, name_component, CORBA_TRUE );
        if ( name == NULL ) {
            printf( "alloc error: new CosNaming::Name()\n" );
            return(1);
        }

        p = "resolve";
        obj = naming_context->resolve( *name, env );
        delete  name;

        CosNotifyChannelAdmin::EventChannel_ptr    channel;
        channel = CosNotifyChannelAdmin::EventChannel::_narrow( obj );
        CORBA::release( obj );

        // ---- connect EventChannel ----
        CosNotifyChannelAdmin::ConsumerAdmin_ptr         admin;
        CosNotifyChannelAdmin::ProxySupplier_ptr         tmp_proxy;
        CosNotifyChannelAdmin::StructuredProxyPullSupplier_ptr     proxy;
        CosNotifyChannelAdmin::ProxyID                   proxyID;

        p = "default_consumer_admin";
        admin = channel->default_consumer_admin( env );
        p = "obtain_notification_pull_supplier";
        tmp_proxy = admin->obtain_notification_pull_supplier(
                        CosNotifyChannelAdmin::STRUCTURED_EVENT, proxyID, env );
        proxy = CosNotifyChannelAdmin::StructuredProxyPullSupplier::_narrow( tmp_proxy );
        CORBA::release( tmp_proxy );
        p = "connect_structured_pull_consumer";
        proxy->connect_structured_pull_consumer(
                        CosNotifyComm::StructuredPullConsumer::_nil(), env );

        // ---- create ES_Util interface object ----
        p = "create_util";
        obj= channel->create_util(
                    CosNotifyChannelAdmin::EventChannel::CHANNEL_UTIL, env );
        util = ES::ChannelUtil::_narrow( obj );
        CORBA::release( obj );

        // ---- local begin ----
        p = "local_begin";
        util->local_begin( proxy,
                           env );

        CosNotification::StructuredEvent    *recv;
        try {
            p = "pull_structured_event";
            recv = proxy->pull_structured_event( env );
        } catch( CORBA::SystemException &se ) {
            printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                    p, se.id(), se.minor() );
            try {
                util->local_rollback( proxy,
                                      tmp_env );
                proxy->disconnect_structured_pull_supplier( env );
            } catch( CORBA::Exception &e ) {}
            return(1);
        } catch( CORBA::Exception &e ) {
            printf( "Operation=%s, Exception=%s\n", p, e.id() );
            try {
                proxy->disconnect_structured_pull_supplier( env );
            } catch( CORBA::Exception &e ) {}
            return(1);
        }

        CORBA::Char *message = 
            (CORBA::Char *)recv->remainder_of_body->value();
        printf( "pull message : %s\n", message );
        delete recv;

        // ---- local commit ----
        p = "local_commit";
        util->local_commit( proxy,
                            env );

        p = "disconnect_structured_pull_supplier";
        proxy->disconnect_structured_pull_supplier( env );

        CORBA::release( proxy );
        CORBA::release( util );
        CORBA::release( admin );
        CORBA::release( naming_context );
        CORBA::release( channel );

    } catch( CORBA::SystemException   &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception   &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    return(0);
}

